@@ -2,9 +2,9 @@
 #
 #     written by G. S. Marzot (marz@users.sourceforge.net)
 #
-#     Copyright (c) 2006-2008 SPARTA, Inc.  All rights reserved.
+#     Copyright (c) 2006-2012 SPARTA, Inc.  All rights reserved.
 #
-#     Copyright (c) 2006-2007 G. S. Marzot. All rights reserved.
+#     Copyright (c) 2006-2009 G. S. Marzot. All rights reserved.
 #
 #
 #     This program is free software; you can redistribute it and/or
@@ -16,7 +16,8 @@ require Net::hostent; # return type from gethost*
 use Net::DNS; # to interpret DNS classes and types
 use Carp;
 
-our $VERSION = '1.30';   # current release version number
+our $VERSION = '1.10';   # current release version number
+our $DNSSECTOOLSVERSION = "DNSSEC-Tools Version: 1.9";
 
 use Exporter;
 use DynaLoader;
@@ -34,66 +35,180 @@ sub AUTOLOAD {
 
 our @ISA = qw(Exporter DynaLoader);
 
+
 our @EXPORT = qw(
- ALG_DH ALG_DSASHA1 ALG_DS_HASH_SHA1 ALG_DS_HASH_SHA256
- ALG_NSEC3_DSASHA1 ALG_NSEC3_HASH_SHA1 ALG_NSEC3_RSASHA1 ALG_RSAMD5
- ALG_RSASHA1 CANNOT_BE_USED CAN_SIGN_KEY CAN_SIGN_ZONE
- CAN_SIGN_ZONE_AND_KEY DNAME_MAX DNS_PORT EDNS_UDP_SIZE ENVELOPE
- MAX_ALIAS_CHAIN_LENGTH MAX_GLUE_FETCH_DEPTH MAX_PROOFS
- NSEC3_FLAG_OPTOUT QUERY_BAD_CACHE_THRESHOLD QUERY_BAD_CACHE_TTL
- Q_ANSWERED Q_CONFLICTING_ANSWERS Q_ERROR_BASE Q_INIT Q_MISSING_GLUE
- Q_QUERY_ERROR Q_REFERRAL_ERROR Q_RESPONSE_ERROR Q_SENT
- Q_WAIT_FOR_GLUE Q_WRONG_ANSWER RRSIGLABEL SHA256_DIGEST_LENGTH
- SHA_DIGEST_LENGTH SIGNBY SR_ANS_BARE_RRSIG SR_ANS_CNAME SR_ANS_DNAME
- SR_ANS_NACK SR_ANS_STRAIGHT SR_ANS_UNSET SR_CALL_ERROR
- SR_CRED_AUTH_ADD SR_CRED_AUTH_ANS SR_CRED_AUTH_AUTH SR_CRED_FILE
- SR_CRED_NONAUTH_ADD SR_CRED_NONAUTH_ANS SR_CRED_NONAUTH_AUTH
- SR_CRED_UNSET SR_DNS_GENERIC_ERROR SR_EDNS_VERSION_ERROR SR_FORMERR
- SR_HEADER_ERROR SR_INTERNAL_ERROR SR_MEMORY_ERROR
- SR_MKQUERY_INTERNAL_ERROR SR_NAME_EXPANSION_FAILURE SR_NOTIMPL
- SR_NO_ANSWER SR_NO_ANSWER_YET SR_NXDOMAIN SR_RCV_INTERNAL_ERROR
- SR_REFUSED SR_SEND_INTERNAL_ERROR SR_SERVFAIL SR_TSIG_ERROR
- SR_TSIG_INTERNAL_ERROR SR_UNSET SR_UNSUPP_EDNS0_LABEL
- SR_ZI_STATUS_LEARNED SR_ZI_STATUS_PERMANENT SR_ZI_STATUS_UNSET TTL
- VALIDATOR_LOG_PORT VAL_AC_ALGORITHM_NOT_SUPPORTED VAL_AC_BAD_BASE
- VAL_AC_BARE_RRSIG VAL_AC_CAN_VERIFY VAL_AC_DATA_MISSING
- VAL_AC_DNSKEY_MISSING VAL_AC_DNSKEY_NOMATCH VAL_AC_DNS_ERROR
- VAL_AC_DONT_GO_FURTHER VAL_AC_DS_MISSING VAL_AC_DS_NOMATCH
- VAL_AC_ERROR_BASE VAL_AC_FAIL_BASE VAL_AC_IGNORE_VALIDATION
- VAL_AC_INIT VAL_AC_INVALID_KEY VAL_AC_INVALID_RRSIG VAL_AC_LAST_BAD
- VAL_AC_LAST_ERROR VAL_AC_LAST_FAILURE VAL_AC_LAST_STATE
- VAL_AC_NEGATIVE_PROOF VAL_AC_NOT_VERIFIED VAL_AC_NO_TRUST_ANCHOR
- VAL_AC_PROVABLY_INSECURE VAL_AC_RRSIG_ALGORITHM_MISMATCH
- VAL_AC_RRSIG_EXPIRED VAL_AC_RRSIG_MISSING VAL_AC_RRSIG_NOTYETACTIVE
- VAL_AC_RRSIG_VERIFIED VAL_AC_RRSIG_VERIFIED_SKEW
- VAL_AC_RRSIG_VERIFY_FAILED VAL_AC_SIGNING_KEY VAL_AC_TRUST
- VAL_AC_TRUSTED_ZONE VAL_AC_TRUST_NOCHK VAL_AC_TRUST_POINT
- VAL_AC_UNKNOWN_ALGORITHM_LINK VAL_AC_UNKNOWN_DNSKEY_PROTOCOL
- VAL_AC_UNSET VAL_AC_UNTRUSTED_ZONE VAL_AC_VERIFIED
- VAL_AC_VERIFIED_LINK VAL_AC_WAIT_FOR_RRSIG VAL_AC_WAIT_FOR_TRUST
- VAL_AC_WCARD_VERIFIED VAL_AC_WCARD_VERIFIED_SKEW
- VAL_AC_WRONG_LABEL_COUNT VAL_BAD_ARGUMENT VAL_BAD_PROVABLY_INSECURE
- VAL_BARE_RRSIG VAL_BARE_TRUST_KEY VAL_BOGUS VAL_BOGUS_PROOF
- VAL_BOGUS_PROVABLE VAL_CONF_NOT_FOUND VAL_CONF_PARSE_ERROR
- VAL_CTX_IDLEN VAL_DNS_ERROR VAL_DONT_KNOW VAL_EACCESS VAL_EINVAL
- VAL_ENOENT VAL_ENOMEM VAL_ENOSYS VAL_FLAG_CHAIN_COMPLETE
- VAL_FROM_ADDITIONAL VAL_FROM_ANSWER VAL_FROM_AUTHORITY VAL_FROM_QUERY
- VAL_FROM_UNSET VAL_IGNORE_VALIDATION VAL_INCOMPLETE_PROOF
- VAL_INTERNAL_ERROR VAL_IRRELEVANT_PROOF VAL_LOCAL_ANSWER
- VAL_MASKED_FLAG_CHAIN_COMPLETE VAL_NONEXISTENT_NAME
- VAL_NONEXISTENT_NAME_NOCHAIN VAL_NONEXISTENT_NAME_OPTOUT
- VAL_NONEXISTENT_TYPE VAL_NONEXISTENT_TYPE_NOCHAIN VAL_NOTRUST
- VAL_NOT_IMPLEMENTED VAL_NO_ERROR VAL_NO_PERMISSION VAL_NO_POLICY
- VAL_OUT_OF_MEMORY VAL_POL_GOPT_DISABLE VAL_POL_GOPT_ENABLE
- VAL_POL_GOPT_OVERRIDE VAL_PROVABLY_INSECURE VAL_QFLAGS_ANY
- VAL_QFLAGS_USERMASK VAL_QUERY_DONT_VALIDATE VAL_QUERY_GLUE_REQUEST
- VAL_QUERY_NO_AC_DETAIL VAL_QUERY_NO_DLV VAL_QUERY_USING_DLV
- VAL_Q_ONLY_MATCHING_FLAGS VAL_RESOURCE_UNAVAILABLE VAL_SUCCESS
- VAL_TRUSTED_ANSWER VAL_TRUSTED_ZONE VAL_UNTRUSTED_ANSWER
- VAL_UNTRUSTED_ZONE VAL_VALIDATED_ANSWER VAL_VERIFIED_CHAIN
- ZONE_USE_NOTHING ZONE_USE_TSIG ns_t_dlv ns_t_dnskey ns_t_ds ns_t_nsec
- ns_t_nsec3 ns_t_rrsig
-	      );
+	LIBSRES_NS_STAGGER
+	LOG_ALERT
+	LOG_CRIT
+	LOG_DEBUG
+	LOG_EMERG
+	LOG_ERR
+	LOG_INFO
+	LOG_NOTICE
+	LOG_WARNING
+	MAX_PROOFS
+	NS_MAXCDNAME
+	ns_msg_getflag
+	ns_t_dlv
+	ns_t_dnskey
+	ns_t_ds
+	ns_t_nsec
+	ns_t_nsec3
+	ns_t_rrsig
+	p_as_error
+	VAL_AC_DS_MISSING
+	p_val_error
+	SOCKET
+	SR_CALL_ERROR
+	SR_DNS_GENERIC_ERROR
+	SR_EDNS_VERSION_ERROR
+	SR_FORMERR
+	SR_HEADER_ERROR
+	SR_INTERNAL_ERROR
+	SR_MEMORY_ERROR
+	SR_MKQUERY_INTERNAL_ERROR
+	SR_NAME_EXPANSION_FAILURE
+	SR_NO_ANSWER
+	SR_NO_ANSWER_YET
+	SR_NOTIMPL
+	SR_NXDOMAIN
+	SR_RCV_INTERNAL_ERROR
+	SR_REFUSED
+	SR_SEND_INTERNAL_ERROR
+	SR_SERVFAIL
+	SR_TSIG_ERROR
+	SR_TSIG_INTERNAL_ERROR
+	SR_UNSET
+	SR_UNSUPP_EDNS0_LABEL
+	SR_ZI_STATUS_LEARNED
+	SR_ZI_STATUS_PERMANENT
+	SR_ZI_STATUS_UNSET
+	VAL_AC_ALGORITHM_NOT_SUPPORTED
+	VAL_AC_BAD_BASE
+	VAL_AC_BARE_RRSIG
+	VAL_AC_CAN_VERIFY
+	VAL_AC_DATA_MISSING
+	VAL_AC_DNS_ERROR
+	VAL_AC_DNSKEY_MISSING
+	VAL_AC_DNSKEY_NOMATCH
+	VAL_AC_DONT_GO_FURTHER
+	VAL_AC_DS_MISSING
+	VAL_AC_DS_NOMATCH
+	VAL_AC_ERROR_BASE
+	VAL_AC_FAIL_BASE
+	VAL_AC_IGNORE_VALIDATION
+	VAL_AC_INIT
+	VAL_AC_INVALID_DS
+	VAL_AC_INVALID_KEY
+	VAL_AC_INVALID_RRSIG
+	VAL_AC_LAST_BAD
+	VAL_AC_LAST_ERROR
+	VAL_AC_LAST_FAILURE
+	VAL_AC_LAST_STATE
+	VAL_AC_NEGATIVE_PROOF
+	VAL_AC_NO_LINK
+	VAL_AC_NOT_VERIFIED
+	VAL_AC_PINSECURE
+	VAL_AC_RRSIG_ALGORITHM_MISMATCH
+	VAL_AC_RRSIG_EXPIRED
+	VAL_AC_RRSIG_MISSING
+	VAL_AC_RRSIG_NOTYETACTIVE
+	VAL_AC_RRSIG_VERIFIED
+	VAL_AC_RRSIG_VERIFIED_SKEW
+	VAL_AC_RRSIG_VERIFY_FAILED
+	VAL_AC_SIGNING_KEY
+	VAL_AC_TRUST
+	VAL_AC_TRUST_ANCHOR
+	VAL_AC_TRUST_NOCHK
+	VAL_AC_TRUST_POINT
+	VAL_AC_UNKNOWN_ALGORITHM_LINK
+	VAL_AC_UNKNOWN_DNSKEY_PROTOCOL
+	VAL_AC_UNSET
+	VAL_AC_UNTRUSTED_ZONE
+	VAL_AC_VERIFIED
+	VAL_AC_VERIFIED_LINK
+	VAL_AC_WAIT_FOR_RRSIG
+	VAL_AC_WAIT_FOR_TRUST
+	VAL_AC_WCARD_VERIFIED
+	VAL_AC_WCARD_VERIFIED_SKEW
+	VAL_AC_WRONG_LABEL_COUNT
+	VAL_AS_CB_COMPLETED
+	VAL_AS_CTX_USER_SUPPLIED
+	VAL_AS_DONE
+	VAL_AS_IGNORE_CACHE
+	VAL_AS_NO_ANSWERS
+	VAL_AS_NO_CALLBACKS
+	VAL_AS_NO_NEW_QUERIES
+	VAL_BAD_ARGUMENT
+	VAL_BARE_RRSIG
+	VAL_BARE_TRUST_KEY
+	VAL_BOGUS
+	VAL_BOGUS_PROOF
+	VAL_BOGUS_PROVABLE
+	VAL_CONF_NOT_FOUND
+	VAL_CONF_PARSE_ERROR
+	VAL_DNS_ERROR
+	VAL_DONT_KNOW
+	VAL_EACCESS
+	VAL_EINVAL
+	VAL_ENOENT
+	VAL_ENOMEM
+	VAL_ENOSYS
+	VAL_FLAG_CHAIN_COMPLETE
+	VAL_FROM_ADDITIONAL
+	VAL_FROM_ANSWER
+	VAL_FROM_AUTHORITY
+	VAL_FROM_QUERY
+	VAL_FROM_UNSET
+	VAL_GETADDRINFO_HAS_STATUS
+	VAL_GETNAMEINFO_HAS_STATUS
+	VAL_IGNORE_VALIDATION
+	VAL_INCOMPLETE_PROOF
+	VAL_INTERNAL_ERROR
+	VAL_IRRELEVANT_PROOF
+	VAL_MASKED_FLAG_CHAIN_COMPLETE
+	VAL_NO_ERROR
+	VAL_NONEXISTENT_NAME
+	VAL_NONEXISTENT_NAME_NOCHAIN
+	VAL_NONEXISTENT_TYPE
+	VAL_NONEXISTENT_TYPE_NOCHAIN
+	VAL_NO_PERMISSION
+	VAL_NO_POLICY
+	VAL_NOT_IMPLEMENTED
+	VAL_NOTRUST
+	VAL_OOB_ANSWER
+	VAL_OUT_OF_MEMORY
+	VAL_PINSECURE
+	VAL_PINSECURE_UNTRUSTED
+	VAL_POL_GOPT_DISABLE
+	VAL_POL_GOPT_ENABLE
+	VAL_POL_GOPT_OVERRIDE
+	VAL_QFLAGS_ANY
+	VAL_QFLAGS_CACHE_MASK
+	VAL_QFLAGS_NOCACHE_MASK
+	VAL_QFLAGS_USERMASK
+	VAL_QUERY_AC_DETAIL
+	VAL_QUERY_DONT_VALIDATE
+	VAL_QUERY_NO_DLV
+	VAL_QUERY_USING_DLV
+	VAL_QUERY_ASYNC
+	VAL_QUERY_NO_EDNS0_FALLBACK
+	VAL_QUERY_SKIP_RESOLVER
+	VAL_QUERY_MARK_FOR_DELETION
+	VAL_QUERY_IGNORE_SKEW
+	VAL_QUERY_RECURSE
+	VAL_QUERY_SKIP_CACHE
+	VAL_QUERY_EDNS0_FALLBACK
+	VAL_QUERY_GLUE_REQUEST
+	VAL_RESOURCE_UNAVAILABLE
+	VAL_SUCCESS
+	VAL_TRUSTED_ANSWER
+	VAL_UNTRUSTED_ANSWER
+	VAL_UNTRUSTED_ZONE
+	VAL_VALIDATED_ANSWER
+	ZONE_USE_NOTHING
+	ZONE_USE_TSIG
+);
 
 bootstrap Net::DNS::SEC::Validator;
 
@@ -213,8 +328,6 @@ sub resolve_and_check {
     $class = Net::DNS::classesbyname($class) unless $class =~ /^\d+$/;
     $type = Net::DNS::typesbyname($type) unless $type =~ /^\d+$/;
 
-    $flags ||= VAL_QUERY_MERGE_RRSETS;
-
     return Net::DNS::SEC::Validator::_resolve_and_check($self,
 							$dname,
 							$class,
