@@ -1,12 +1,12 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
  *
  * A command-line tool for testing the val_getaddrinfo() function.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include <validator/validator.h>
 
 #ifdef HAVE_GETOPT_H
@@ -168,10 +168,8 @@ _callback(void *callback_data, int eai_retval, struct addrinfo *res,
     *gas->vstatus = val_status;
     gas->done = 1;
 
-    /*
-    printf("_callback %p %d %p %d\n", callback_data, eai_retval, res,
+    val_log(NULL, LOG_DEBUG, "_callback %p %d %p %d\n", callback_data, eai_retval, res,
            val_status);
-    */
 
     return 0; /* OK */
 }
@@ -307,6 +305,7 @@ main(int argc, char *argv[])
             tv.tv_usec = 567;
             val_async_check_wait(context, NULL, NULL, &tv, 0);
         }
+        val_free_context(context);
 #endif
     }
 
