@@ -92,7 +92,7 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "res_support.h"
@@ -159,7 +159,7 @@ _print_or_log(void *file, int level, const char *template, ...)
     va_start(ap, template);
 
     if (NULL == file)
-        res_log((val_context_t*)NULL, level, template, ap);
+        res_log_ap((val_context_t*)NULL, level, template, ap);
     else
         vfprintf(file, template, ap);
 
@@ -319,7 +319,7 @@ p_fqname(const u_char * cp, const u_char * msg, FILE * file)
  * And, for once, solaris has a better header than the rest, and has const
  * char ptrs in res_sym.
  */
-#if (defined(__APPLE__) || defined(sun) || defined(__CYGWIN__) || defined(linux))
+#if (defined(__APPLE__) || defined(sun) || defined(__CYGWIN__) || defined(linux)) && !defined(ANDROID)
 #define RES_SYM_TYPE res_sym
 #else
 struct res_sym_const {
@@ -602,54 +602,14 @@ p_option(P_OPTION_ARG_TYPE option)
     static char     nbuf[40];
 
     switch (option) {
-    case RES_INIT:
-        return "init";
-    case RES_DEBUG:
-        return "debug";
-    case RES_AAONLY:
-        return "aaonly(unimpl)";
-    case RES_USEVC:
-        return "usevc";
-    case RES_PRIMARY:
-        return "primry(unimpl)";
-    case RES_IGNTC:
-        return "igntc";
-    case RES_RECURSE:
+    case SR_QUERY_SET_DO:
+        return "DO-bit";
+    case SR_QUERY_SET_CD:
+        return "CD-bit";
+    case SR_QUERY_RECURSE:
         return "recurs";
-    case RES_DEFNAMES:
-        return "defnam";
-    case RES_STAYOPEN:
-        return "styopn";
-    case RES_DNSRCH:
-        return "dnsrch";
-    case RES_INSECURE1:
-        return "insecure1";
-    case RES_INSECURE2:
-        return "insecure2";
-    case RES_NOALIASES:
-        return "noaliases";
-    case RES_USE_INET6:
-        return "inet6";
-#ifdef RES_USE_EDNS0            /* KAME extension */
-    case RES_USE_EDNS0:
-        return "edns0";
-#endif
-#ifdef RES_USE_DNAME
-    case RES_USE_DNAME:
-        return "dname";
-#endif
-#ifdef RES_USE_DNSSEC
-    case RES_USE_DNSSEC:
-        return "dnssec";
-#endif
-#ifdef RES_NOTLDQUERY
-    case RES_NOTLDQUERY:
-        return "no-tld-query";
-#endif
-#ifdef RES_NO_NIBBLE2
-    case RES_NO_NIBBLE2:
-        return "no-nibble2";
-#endif
+    case SR_QUERY_DEBUG:
+        return "debug";
         /*
          * XXX nonreentrant 
          */
