@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "res_support.h"
@@ -30,20 +30,6 @@
 #define FALSE 0
 #endif
 
-static int res_io_debug = FALSE;
-
-void
-res_io_set_debug(int val)
-{
-    res_io_debug = val;
-}
-
-int
-res_io_get_debug(void)
-{
-    return res_io_debug;
-}
-
 /*
  * Less than or equal comparison for timeval structures
  * note: timercmp doesn't handle <= or >=
@@ -92,27 +78,52 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
  * Find a port in the range 1024 - 65535 
  */
 static int
-bind_to_random_source(SOCKET s)
+bind_to_random_source(int af, SOCKET s)
 {   
-    /* XXX This is not IPv6 ready yet. Probably need to have
-     * XXX a config statement in dnsval.conf that allows the
-     * XXX user to specify the preferred source IP
-     */
     struct sockaddr_storage ea_source;
-    struct sockaddr_in *sa = (struct sockaddr_in *) &ea_source;
+    struct sockaddr_in *sa4 = (struct sockaddr_in *) &ea_source;
+#ifdef VAL_IPV6
+    struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *) &ea_source;
+#endif
+    struct sockaddr *sa;
+    size_t sock_size;
+
     u_int16_t next_port, start_port;
 
-    memset(sa, 0, sizeof(ea_source));
-    sa->sin_family = AF_INET;
-    sa->sin_addr.s_addr = htonl(INADDR_ANY);
+    memset(&ea_source, 0, sizeof(ea_source));
+    if (af == AF_INET) {
+        sa4->sin_family = AF_INET;
+        sa4->sin_addr.s_addr = htonl(INADDR_ANY);
+#ifdef VAL_IPV6
+    } else if (af == AF_INET6) {
+        sa6->sin6_family = AF_INET6;
+        /*struct in6_addr anyaddr = IN6ADDR_ANY_INIT*/
+        /*sa6->sin6_addr = IN6ADDR_ANY_INIT*/
+        sa6->sin6_addr = in6addr_any;
+#endif
+    } else {
+        res_log(NULL,LOG_ERR,"libsres: could not bind to random port for unsupported address family %d", af);
+        return 1; /* failure */
+    }
 
     start_port = (libsres_random() % 64512) + 1024;
     next_port = start_port;
 
     do {
-        sa->sin_port = htons(next_port);
-        if (0 == bind(s, (const struct sockaddr *)sa, 
-                    sizeof(struct sockaddr_in))) {
+        if (af == AF_INET) {
+            sa4->sin_port = htons(next_port);
+            sa = (struct sockaddr *) sa4;
+            sock_size = sizeof(struct sockaddr_in);
+#ifdef VAL_IPV6
+        } else { /* AF_INET6 */
+            sa6->sin6_port = htons(next_port);
+            sa = (struct sockaddr *) sa6;
+            sock_size = sizeof(struct sockaddr_in6);
+#endif
+        }
+
+        if (0 == bind(s, (const struct sockaddr *)sa, sock_size)) {
+            //res_log(NULL,LOG_ERR,"libsres: bound to random port %d", next_port);
             return 0; /* success */
         } else  {
             /* error */
@@ -124,7 +135,7 @@ bind_to_random_source(SOCKET s)
     } while (next_port != start_port);
 
     /* wrapped around and still no ports found */
-    res_log(NULL,LOG_ERR,"libsres: ""could not bind to random port");
+    res_log(NULL,LOG_ERR,"libsres: could not bind to random port above %d", start_port);
 
     return 1; /* failure */
 }
@@ -149,6 +160,7 @@ res_sq_free_expected_arrival(struct expected_arrival **ea)
 {
     if ((ea == NULL) || (*ea == NULL))
         return;
+
     if ((*ea)->ea_socket != INVALID_SOCKET)
         res_log(NULL, LOG_DEBUG, "libsres: ""ea %p, fd %d free",
                 *ea, (*ea)->ea_socket);
@@ -157,13 +169,27 @@ res_sq_free_expected_arrival(struct expected_arrival **ea)
                 *ea, (*ea)->ea_socket);
     if ((*ea)->ea_ns != NULL)
         free_name_server(&((*ea)->ea_ns));
+#ifdef EA_EXTRA_DEBUG
+    if ((*ea)->name != NULL)
+        free((*ea)->name);
+#endif
     if ((*ea)->ea_socket != INVALID_SOCKET)
         CLOSESOCK((*ea)->ea_socket);
     if ((*ea)->ea_signed)
         FREE((*ea)->ea_signed);
     if ((*ea)->ea_response)
         FREE((*ea)->ea_response);
+
+#ifdef DEBUG_DONT_RELEASE_ANYTHING
+    {
+        static struct expected_arrival *holding = NULL;
+
+        (*ea)->ea_next = holding;
+        holding = *ea;
+    }
+#else
     FREE(*ea);
+#endif
 
     *ea = NULL;
 }
@@ -186,7 +212,15 @@ set_alarm(struct timeval *tv, long delay)
 {
     gettimeofday(tv, NULL);
     tv->tv_sec += delay;
-    tv->tv_usec = 0;
+}
+
+void
+set_alarms(struct expected_arrival *ea, long next, long cancel)
+{
+    gettimeofday(&ea->ea_next_try, NULL);
+    ea->ea_next_try.tv_sec += next;
+    ea->ea_cancel_time.tv_sec = ea->ea_next_try.tv_sec + cancel;
+    ea->ea_cancel_time.tv_usec = ea->ea_next_try.tv_usec;
 }
 
 struct expected_arrival *
@@ -205,6 +239,7 @@ res_ea_init(u_char * signed_query, size_t signed_length,
     memset(temp, 0x0, sizeof(struct expected_arrival));
     temp->ea_socket = INVALID_SOCKET;
     temp->ea_ns = ns;
+    temp->ea_edns0_size = ns->ns_edns0_size;
     temp->ea_which_address = 0;
     temp->ea_using_stream = FALSE;
     temp->ea_signed = signed_query;
@@ -212,8 +247,7 @@ res_ea_init(u_char * signed_query, size_t signed_length,
     temp->ea_response = NULL;
     temp->ea_response_length = 0;
     temp->ea_remaining_attempts = ns->ns_retry+1;
-    set_alarm(&temp->ea_next_try, delay);
-    set_alarm(&temp->ea_cancel_time, delay + res_get_timeout(ns));
+    set_alarms(temp, delay, res_get_timeout(ns));
     temp->ea_next = NULL;
 
     return temp;
@@ -239,7 +273,7 @@ res_io_cancel_source(struct expected_arrival *ea)
     }
 
     /* no more retries */
-    ea->ea_remaining_attempts = 0;
+    ea->ea_remaining_attempts = -1;
 
     /* bump cancel time to current time */
     gettimeofday(&ea->ea_cancel_time, NULL);
@@ -300,18 +334,18 @@ res_io_send(struct expected_arrival *shipit)
      * which causes the source to be cancelled next go-round.
      */
     if (shipit->ea_socket == INVALID_SOCKET) {
-        int             i = shipit->ea_which_address;
+        int i = shipit->ea_which_address;
+        int af =  shipit->ea_ns->ns_address[i]->ss_family;
 
-        shipit->ea_socket = socket(shipit->ea_ns->ns_address[i]->ss_family,
-                                   socket_type, 0);
+        shipit->ea_socket = socket(af, socket_type, 0);
         if (shipit->ea_socket == INVALID_SOCKET) {
-            res_log(NULL,LOG_ERR,"libsres: ""socket() failed, errno = %d",
-                errno);
+            res_log(NULL,LOG_ERR,"libsres: ""socket() failed, errno = %d %s",
+                    errno, strerror(errno));
             return SR_IO_SOCKET_ERROR;
         }
 
         /* Set the source port */
-        if (0 != bind_to_random_source(shipit->ea_socket)) {
+        if (0 != bind_to_random_source(af, shipit->ea_socket)) {
             /* error */
             CLOSESOCK(shipit->ea_socket);
             return SR_IO_SOCKET_ERROR;
@@ -320,10 +354,17 @@ res_io_send(struct expected_arrival *shipit)
         /*
          * OS X wants the socket size to be sockaddr_in for INET,
          * while Linux is happy with sockaddr_storage. 
-         * XXX Might need to fix this for sockaddr_in6 too...
          */
-        socket_size = shipit->ea_ns->ns_address[i]->ss_family == AF_INET ?
-                      sizeof(struct sockaddr_in) : sizeof(struct sockaddr_storage);
+        if (af == AF_INET) {
+            socket_size = sizeof(struct sockaddr_in);
+#ifdef VAL_IPV6
+        } else if (af == AF_INET6) {
+            socket_size = sizeof(struct sockaddr_in6);
+#endif
+        } else {
+            socket_size = sizeof(struct sockaddr_storage); 
+        }
+
         if (connect
             (shipit->ea_socket,
              (struct sockaddr *) shipit->ea_ns->ns_address[i],
@@ -378,103 +419,48 @@ res_io_send(struct expected_arrival *shipit)
     delay = shipit->ea_ns->ns_retrans
         << (shipit->ea_ns->ns_retry + 1 - shipit->ea_remaining_attempts--);
     res_log(NULL, LOG_DEBUG, "libsres: ""next try delay %d", delay);
-    set_alarm(&shipit->ea_next_try, delay);
+    set_alarms(shipit, delay, res_get_timeout(shipit->ea_ns));
     res_print_ea(shipit);
 
     return SR_IO_UNSET;
 }
 
+/*
+ * 1 : edns0 fallback succeeded, ready for retry.
+ * 0 : edns0 fallback failed, no more retries for server
+ * -1: unknown error
+ *     NOTE: val_res_nsfallback will cancel an entire request if -1 returned
+ */
 int 
 res_nsfallback(int transaction_id, struct timeval *closest_event, 
-               const char *name, const u_int16_t class_h, 
-               const u_int16_t type_h, int *edns0)
+               struct name_server *server, const char *name,
+               const u_int16_t class_h, const u_int16_t type_h)
 {
     struct expected_arrival *temp;
     int ret_val = -1;
 
-    if (transaction_id == -1 )
+    if (transaction_id < 0)
         return -1;
 
     pthread_mutex_lock(&mutex);
-    for (temp = transactions[transaction_id];
-         temp && temp->ea_remaining_attempts == -1;
-         temp = temp->ea_next)
-         ;
+    temp = transactions[transaction_id];
     if (temp != NULL)
-        ret_val = res_nsfallback_ea(temp, closest_event, name, class_h, type_h,
-                                    edns0);
-    else
-        *edns0 = 0;
-
+        ret_val = res_nsfallback_ea(temp, closest_event, server, name, class_h,
+                                    type_h);
     pthread_mutex_unlock(&mutex);
     return ret_val;
 }
 
-int 
-res_nsfallback_ea(struct expected_arrival *temp, struct timeval *closest_event, 
-                  const char *name, const u_int16_t class_h, 
-                  const u_int16_t type_h, int *edns0)
+static void
+_reset_timeouts(struct expected_arrival *temp)
 {
-    const static int edns0_fallback[] = { 4096, 1492, 512, 0 };
-    long             delay = 0, i;
+    res_log(NULL, LOG_INFO, "libsres: ""reset timeout for %p", temp);
 
-    if (!temp && !name)
-        return -1;
+    set_alarms(temp, 0, res_get_timeout(temp->ea_ns));
 
-    if ((temp->ea_ns->ns_options & RES_USE_DNSSEC) && 
-        (temp->ea_ns->ns_edns0_size > 0)) {
-        *edns0 = 1;
-        for (i = 0; i < sizeof(edns0_fallback); i++) {
-            if (temp->ea_ns->ns_edns0_size > edns0_fallback[i]) {
-                /* try using a lower edns0 value */
-                temp->ea_ns->ns_edns0_size = edns0_fallback[i];
-                if (edns0_fallback[i] == 0) {
-                    /* try without EDNS0 */
-                    temp->ea_ns->ns_options ^= RES_USE_DNSSEC;
-                    *edns0 = 0;
-                    if (temp->ea_signed)
-                        FREE(temp->ea_signed);
-                    temp->ea_signed = NULL;
-                    temp->ea_signed_length = 0;
-
-                    if (res_create_query_payload(temp->ea_ns,
-                                    name, class_h, type_h,
-                                    &temp->ea_signed,
-                                    &temp->ea_signed_length) < 0)
-                        break;
-                }
-                temp->ea_remaining_attempts++;
-                if (temp->ea_socket != INVALID_SOCKET)
-                    CLOSESOCK(temp->ea_socket);
-                temp->ea_socket = INVALID_SOCKET;
-
-                break;
-            }
-        }
-    }
-    else
-        *edns0 = 0;
-
-    res_log(NULL, LOG_INFO, "libsres: "
-            "ns fallback for {%s %s(%d) %s(%d)}, edns0: %d/%d",
-            name, p_class(class_h), class_h, p_type(type_h), type_h,
-            *edns0, temp->ea_ns->ns_edns0_size);
-
-
-    if (temp->ea_remaining_attempts == 0) {
-        res_log(NULL, LOG_DEBUG, "libsres: ""no remaining attempts");
-        return -1;
-    }
-
-    gettimeofday(&temp->ea_next_try, NULL);
-    for (i = 0; i < temp->ea_remaining_attempts; i++)
-        delay += temp->ea_ns->ns_retrans << i;
-
-    set_alarm(&temp->ea_cancel_time, delay); 
-    UPDATE(closest_event, temp->ea_next_try);
     /* 
      *  if next event is in the future, make sure we
-     *  offset it to the current time 
+     *  offset it to the current time IF it's not already active.
      */
     if (temp->ea_next) {
         struct expected_arrival *t;
@@ -482,12 +468,130 @@ res_nsfallback_ea(struct expected_arrival *temp, struct timeval *closest_event,
                             temp->ea_next_try.tv_sec;
         if (offset > 0) {
             for (t=temp->ea_next; t; t=t->ea_next) {
+                if (INVALID_SOCKET != t->ea_socket)
+                    continue;
+                res_log(NULL, LOG_INFO, "libsres: ""timeout offset %ld for %p",
+                        offset, t);
                 t->ea_next_try.tv_sec -= offset;
                 t->ea_cancel_time.tv_sec -= offset;
             } 
         }
     }
-    return 0;
+}
+
+/*
+ * 1 : edns0 fallback succeeded, ready for retry.
+ * 0 : edns0 fallback failed, no more retries for server
+ * -1: unknown error
+ *     NOTE: val_res_nsfallback will cancel an entire request if -1 returned
+ */
+int 
+res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event, 
+                  struct name_server *server, const char *name,
+                  const u_int16_t class_h, const u_int16_t type_h)
+{
+    const static int edns0_fallback[] = { 4096, 1492, 512, 0 };
+    long             i, old_size;
+    struct expected_arrival *temp = ea;
+
+    if (!temp && !name)
+        return -1;
+
+    if (!server) {
+        res_log(NULL, LOG_DEBUG, "libsres: ""no server specified");
+        // return -1;
+    }
+
+    for(;temp;temp=temp->ea_next) {
+        //res_print_ea(temp);
+        /** match name, then look for address */
+        if (namecmp(server->ns_name_n, temp->ea_ns->ns_name_n) != 0)
+            continue;
+        if (memcmp(server->ns_address[0],
+                   temp->ea_ns->ns_address[temp->ea_which_address],
+                   sizeof(*server->ns_address[0])) == 0)
+            break;
+    }
+
+    if (!temp) {
+        res_log(NULL, LOG_DEBUG, "libsres: "
+                "no matching server found for fallback");
+        return -1;
+    }
+
+    /** even if there is a smaller size to fall back to, no attempts left */
+    if (temp->ea_remaining_attempts < 0) {
+        res_log(NULL, LOG_DEBUG, "libsres: "
+                "ea %p no remaining attempts for fallback", temp);
+        if (res_io_are_all_finished(ea))
+            return -1;
+        return 0;
+    }
+
+    res_log(NULL, LOG_DEBUG, "libsres: ""ea %p attempting ns fallback", temp);
+
+    old_size = temp->ea_edns0_size;
+    if ((temp->ea_ns->ns_options & SR_QUERY_VALIDATING_STUB_FLAGS) && 
+        (temp->ea_edns0_size > 0)) {
+        for (i = 0; i < sizeof(edns0_fallback); i++) {
+            if (temp->ea_edns0_size > edns0_fallback[i]) {
+                /* try using a lower edns0 value */
+                temp->ea_edns0_size = edns0_fallback[i];
+                if (edns0_fallback[i] == 0) {
+                    /* try without EDNS0 */
+                    res_log(NULL, LOG_DEBUG, "libsres: "
+                            "fallback disabling edns0");
+                    temp->ea_ns->ns_options ^= SR_QUERY_VALIDATING_STUB_FLAGS;
+                }
+                temp->ea_remaining_attempts++;
+                break;
+            }
+        }
+    }
+
+    /** didn't find a smaller size to try and were already on last attempt */
+    if (temp->ea_remaining_attempts == 0) {
+        res_log(NULL, LOG_DEBUG, "libsres: "
+                "fallback already exhausted edns retries");
+        res_io_cancel_source(temp);
+        if (res_io_are_all_finished(ea))
+            return -1;
+        return 0;
+    }
+
+    /** reset timeouts for immediate retries */
+    _reset_timeouts(temp);
+    UPDATE(closest_event, temp->ea_next_try);
+
+    if (0 == old_size) {
+        res_log(NULL, LOG_DEBUG, "libsres: ""fallback already disabled edns");
+        return 0;
+    }
+
+    if (temp->ea_signed)
+        FREE(temp->ea_signed);
+    temp->ea_signed = NULL;
+    temp->ea_signed_length = 0;
+
+    if (res_create_query_payload(temp->ea_ns,
+                name, class_h, type_h,
+                &temp->ea_signed,
+                &temp->ea_signed_length) < 0) {
+        res_log(NULL, LOG_DEBUG, "libsres: ""could not create query payload");
+        return -1;
+    }
+
+    /** close socket so retry uses different port */
+    if (temp->ea_socket != INVALID_SOCKET)
+        CLOSESOCK(temp->ea_socket);
+    temp->ea_socket = INVALID_SOCKET;
+
+    res_log(NULL, LOG_INFO, "libsres: "
+            "ns fallback for {%s %s(%d) %s(%d)}, edns0 size %d > %d",
+            name, p_class(class_h), class_h, p_type(type_h), type_h,
+            old_size, temp->ea_edns0_size);
+
+    return 1;
 }
 
 static void
@@ -506,26 +610,43 @@ res_io_next_address(struct expected_arrival *ea,
             ea->ea_socket = INVALID_SOCKET;
         }
         ea->ea_which_address++;
+        ea->ea_edns0_size = ea->ea_ns->ns_edns0_size;
         ea->ea_remaining_attempts = ea->ea_ns->ns_retry+1;
-        set_alarm(&(ea->ea_next_try), 0);
-        set_alarm(&(ea->ea_cancel_time),res_get_timeout(ea->ea_ns));
+        set_alarms(ea, 0, res_get_timeout(ea->ea_ns));
         res_log(NULL, LOG_INFO,
                 "libsres: ""%s - SWITCHING TO NEW ADDRESS", more_prefix);
     } else {
+        struct expected_arrival *next_server = ea->ea_next;
+
         /*
          * cancel this source 
          */
         res_io_cancel_remaining_attempts(ea);
         res_log(NULL, LOG_INFO, "libsres: ""%s", no_more_str);
+
+        if (next_server && next_server->ea_remaining_attempts &&
+            INVALID_SOCKET == next_server->ea_socket) {
+            res_log(NULL, LOG_INFO, "libsres: ""jump-starting next server %p",
+                    next_server);
+            _reset_timeouts(next_server);
+        }
     }
     res_print_ea(ea);
 }
-
+/*
+ * net_change : optional pointer for returning the next change in the
+ *              number of open/active sockets.
+ *
+ * active : optional pointer for returning number of active queries.
+ *          this includes any query with remaining retries, regardless
+ *          of whether or not it has a current open/active socket.
+ */
 int
 res_io_check_ea_list(struct expected_arrival *ea, struct timeval *next_evt,
                      struct timeval *now, int *net_change, int *active)
 {
     struct timeval  local_now;
+    int             remaining = 0, no_sock = 0, no_att = 0, open = 0;
     
     /*
      * if caller didn't pass us current time, get it
@@ -549,18 +670,22 @@ res_io_check_ea_list(struct expected_arrival *ea, struct timeval *next_evt,
             res_log(NULL, LOG_DEBUG, "libsres: "
                     " skipping %p (sock %d, rem %d)",
                     ea, ea->ea_socket, ea->ea_remaining_attempts);
+            ++no_att;
             continue;
         }
         if (ea->ea_socket != INVALID_SOCKET )
             res_print_ea(ea);
+        else
+            ++no_sock;
 
         /*
          * check for timeouts. If there is another address, move to it
          */
-        if ( LTEQ(ea->ea_cancel_time, (*now))) {
-            res_io_next_address(ea, "TIMEOUTS", "TIMEOUT - CANCELING");
-            if (net_change)
+        if ( LTEQ(ea->ea_cancel_time, (*now)) ||
+             ((0 == ea->ea_remaining_attempts) && LTEQ(ea->ea_next_try, (*now)))) {
+            if (net_change && ea->ea_socket != INVALID_SOCKET)
                 --(*net_change);
+            res_io_next_address(ea, "TIMEOUTS", "TIMEOUT - CANCELING");
         }
 
         /*
@@ -578,46 +703,49 @@ res_io_check_ea_list(struct expected_arrival *ea, struct timeval *next_evt,
                     if (needed_new_socket) {
                         if (net_change)
                             ++(*net_change);
-                        res_print_ea(ea);
                     }
                     break; /* from while remaining attempts */
                 }
             } /* while */
         }
-        else if (ea->ea_socket == INVALID_SOCKET) {
-            struct expected_arrival *tmp_ea = ea;
-#if 0 
-            for (tmp_ea = ea->ea_next; tmp_ea; tmp_ea = tmp_ea->ea_next ) {
-                res_log(NULL, LOG_DEBUG, "libsres: "" skipping ea");
-                res_print_ea(ea);
-            }
-#else
-            int count = 1;
-            for (tmp_ea = ea->ea_next; tmp_ea; tmp_ea = tmp_ea->ea_next )
-                ++count;
-            res_log(NULL, LOG_DEBUG, "libsres: "" skipping remaining %d ns list",
-                    count);
-#endif
-            break;
-        }
 
         /*
          * update next event
          */
         if (ea->ea_remaining_attempts != -1) {
+            ++remaining;
             if (next_evt) {
                 UPDATE(next_evt, ea->ea_cancel_time);
                 UPDATE(next_evt, ea->ea_next_try);
             }
-            if (active)
-                ++(*active);
+            if (ea->ea_socket != INVALID_SOCKET)
+                ++open;
         }
     }
-    if (next_evt)
-        res_log(NULL, LOG_DEBUG, "libsres: ""  Next event %ld.%ld",
-                next_evt->tv_sec, next_evt->tv_usec);
+    if (next_evt) {
+        struct timeval  now,when;
+        gettimeofday(&now, NULL);
+        timersub(next_evt, &now, &when);
+        if (when.tv_sec < 0) {
+            when.tv_sec = when.tv_usec = 0;
+        }
+        res_log(NULL, LOG_DEBUG, "libsres: "
+                "  Next event %ld.%ld (%ld.%ld) remaining %d (%d active)",
+                next_evt->tv_sec, next_evt->tv_usec, when.tv_sec, when.tv_usec,
+                remaining, open);
+    } else
+        res_log(NULL, LOG_DEBUG, "libsres: ""  remaining %d (%d active)",
+                remaining, open);
+    if (no_sock || no_att)
+        res_log(NULL, LOG_DEBUG, "libsres: "
+                "  skipped %d closed sockets, %d no attempts", no_sock, no_att);
 
-    return SR_IO_UNSET;
+    if (active)
+        *active = remaining;
+    if (remaining)
+        return SR_IO_UNSET;
+    else
+        return SR_IO_NO_ANSWER;
 }
 
 int
@@ -729,11 +857,10 @@ res_io_deliver(int *transaction_id, u_char * signed_query,
 }
 
 int
-res_io_queue(int *transaction_id, u_char * signed_query,
-             size_t signed_length, struct name_server *ns, long delay)
+res_io_queue_ea(int *transaction_id, struct expected_arrival *new_ea)
 {
     int             try_index;
-    struct expected_arrival *temp, *new_ea;
+    struct expected_arrival *temp;
 
     /*
      * Determine (new) transaction location 
@@ -766,12 +893,6 @@ res_io_queue(int *transaction_id, u_char * signed_query,
     /*
      * Register this request 
      */
-    new_ea = res_ea_init(signed_query, signed_length, ns, delay);
-    if (new_ea == NULL) {
-        /** We can't add this */
-        pthread_mutex_unlock(&mutex);
-        return SR_IO_MEMORY_ERROR;
-    }
     if (transactions[*transaction_id] == NULL) {
         /*
          * Add this as the first request 
@@ -792,6 +913,26 @@ res_io_queue(int *transaction_id, u_char * signed_query,
     return SR_IO_UNSET;
 }
 
+int
+res_io_queue(int *transaction_id, u_char * signed_query,
+             size_t signed_length, struct name_server *ns, long delay)
+{
+    struct expected_arrival *new_ea;
+    int                      ret_val;
+
+    new_ea = res_ea_init(signed_query, signed_length, ns, delay);
+    if (new_ea == NULL)
+        return SR_IO_MEMORY_ERROR;
+
+    ret_val = res_io_queue_ea(transaction_id, new_ea);
+    if (ret_val != SR_IO_UNSET) {
+        res_free_ea_list(new_ea);
+        return ret_val;
+    }
+
+    return SR_IO_UNSET;
+}
+
 void
 res_io_set_timeout(struct timeval *timeout, struct timeval *next_event)
 {
@@ -832,13 +973,15 @@ void
 res_io_select_info(struct expected_arrival *ea_list, int *nfds,
                    fd_set * read_descriptors, struct timeval *timeout)
 {
-    struct timeval now;
+    struct timeval now, orig;
+    int            count = 0, skipped = 0;
 
     if (timeout) {
         res_log(NULL, LOG_DEBUG,
                 "libsres: "" ea %p select/timeout info", ea_list);
         res_log(NULL, LOG_DEBUG+1, "libsres: ""    orig timeout %ld,%ld",
                 timeout->tv_sec, timeout->tv_usec);
+        memcpy(&orig, timeout, sizeof(orig));
         gettimeofday(&now, NULL);
     }
     else
@@ -850,10 +993,30 @@ res_io_select_info(struct expected_arrival *ea_list, int *nfds,
      */
     for ( ; ea_list; ea_list = ea_list->ea_next) {
         if ((ea_list->ea_remaining_attempts == -1) ||
-            (ea_list->ea_socket == INVALID_SOCKET))
+            (ea_list->ea_socket == INVALID_SOCKET)) {
+            if (ea_list->ea_remaining_attempts > 0) {
+                if (timeout) {
+                    UPDATE(timeout, ea_list->ea_cancel_time);
+                    UPDATE(timeout, ea_list->ea_next_try);
+                }
+                ++skipped;
+            }
+            res_log(NULL,LOG_DEBUG+1, "libsres:""   fd %d, rem %d",
+                    ea_list->ea_socket, ea_list->ea_remaining_attempts);
             continue;
+        }
 
-        res_log(NULL,LOG_DEBUG, "libsres:""   fd %d added", ea_list->ea_socket);
+        if (read_descriptors &&
+            FD_ISSET(ea_list->ea_socket, read_descriptors)) {
+            ++skipped;
+            res_log(NULL,LOG_DEBUG+1, "libsres:""   fd %d already set",
+                    ea_list->ea_socket);
+            continue;
+        }
+                
+        ++count;
+        res_log(NULL,LOG_DEBUG, "libsres:""   fd %d added, rem %d",
+                ea_list->ea_socket, ea_list->ea_remaining_attempts);
         if (read_descriptors)
             FD_SET(ea_list->ea_socket, read_descriptors);
         if (nfds && (ea_list->ea_socket >= *nfds))
@@ -864,10 +1027,16 @@ res_io_select_info(struct expected_arrival *ea_list, int *nfds,
             UPDATE(timeout, ea_list->ea_next_try);
         }
     }
-    if (timeout) {
-        res_log(NULL, LOG_DEBUG, "libsres: ""    final timeout %ld,%ld",
-                timeout->tv_sec, timeout->tv_usec);
+    if (timeout && (orig.tv_sec != timeout->tv_sec ||
+                    orig.tv_usec != timeout->tv_usec)) {
+        res_log(NULL, LOG_DEBUG,
+                "libsres: ""    new timeout %ld.%ld, %d fds added, %d inactive",
+                timeout->tv_sec, timeout->tv_usec, count, skipped);
     }
+    else
+        res_log(NULL, LOG_DEBUG,
+                "libsres: ""    %d fds added, %d inactive", count, skipped);
+
 }
 
 static int
@@ -910,7 +1079,7 @@ res_io_select_sockets(fd_set * read_descriptors, struct timeval *timeout)
 #ifdef HAVE_PSELECT
     struct timespec timeout_ts;
     timeout_ts.tv_sec = timeout->tv_sec;
-    timeout_ts.tv_nsec = timeout->tv_usec;
+    timeout_ts.tv_nsec = timeout->tv_usec * 1000;
     ready = pselect(max_sock + 1, read_descriptors, NULL, NULL, &timeout_ts, NULL);
 #else
     ready = select(max_sock + 1, read_descriptors, NULL, NULL, timeout);
@@ -931,7 +1100,6 @@ wait_for_res_data(fd_set * pending_desc, struct timeval *closest_event)
     int            ready;
 
     res_log(NULL,LOG_DEBUG,"libsres: ""wait_for_res_data");
-
     /*
      * Set the timeout in case nothing arrives.  The timeout will expire
      * prior to the next event that res_io_check needs to initiate.  If
@@ -951,57 +1119,104 @@ wait_for_res_data(fd_set * pending_desc, struct timeval *closest_event)
     // will catch this condition when we actually read data
 }
 
+static int
+_clone_respondent(struct expected_arrival *ea,
+                  struct name_server **respondent)
+{
+    int save_count = -1, retval;
+
+    /*
+     * don't clone all when we just need one. temporarily set
+     * number of nameservers to 1 before cloning.
+     */
+    if (ea->ea_ns->ns_number_of_addresses > 1) {
+        save_count = ea->ea_ns->ns_number_of_addresses;
+        ea->ea_ns->ns_number_of_addresses = 1;
+    }
+    if (SR_UNSET != (retval = clone_ns(respondent, ea->ea_ns)))
+        return retval;
+    if (save_count > 0) /* restore original count */
+        ea->ea_ns->ns_number_of_addresses = save_count;
+    
+    /** if response wasn't from first address, fixup respondent */
+    if (ea->ea_which_address != 0) {
+        memcpy(((*respondent)->ns_address[0]),
+               ea->ea_ns->ns_address[ea->ea_which_address],
+               sizeof(struct sockaddr_storage));
+    }
+
+    return SR_UNSET;
+}
+
 int
 res_io_get_a_response(struct expected_arrival *ea_list, u_char ** answer,
                       size_t * answer_length,
                       struct name_server **respondent)
 {
-    int             retval;
-    int             save_count = -1;
+    int             retval = SR_IO_UNSET, retries = 0;
 
-    res_log(NULL,LOG_DEBUG,"libsres: "" checking for response for ea %p",
+    struct expected_arrival *orig = ea_list;
+    res_log(NULL,LOG_DEBUG,"libsres: "" checking for response for ea %p list",
             ea_list);
     for( ; ea_list; ea_list = ea_list->ea_next) {
-        if (ea_list->ea_remaining_attempts == -1)
-            continue;
+
+        if (ea_list->ea_remaining_attempts != -1)
+            ++retries;
 
         if (!ea_list->ea_response)
             continue;
 
+        if (ea_list->ea_remaining_attempts == -1) {
+            res_log(NULL, LOG_DEBUG, "libsres: "
+                    " *** ANSWER with no remaining attempts");
+            //continue;
+        }
+
+        /** basic format checks; NOTE: returns SR_*, *NOT* SR_IO_* */
+        retval = res_response_checks(&ea_list->ea_response,
+                                     &ea_list->ea_response_length, respondent);
+        if (SR_UNSET != retval) { /* cleared response */
+            res_log(NULL, LOG_DEBUG, "libsres: "
+                    "*** dropped response for ea %p rc %d", ea_list, retval);
+            /** close socket so retry uses different port */
+            if (ea_list->ea_socket != INVALID_SOCKET) {
+                CLOSESOCK (ea_list->ea_socket);
+                ea_list->ea_socket = INVALID_SOCKET;
+            }
+            res_print_ea(ea_list);
+            _clone_respondent(ea_list, respondent);
+            set_alarms(ea_list, 0, res_get_timeout(ea_list->ea_ns));
+            retval = SR_IO_NO_ANSWER;
+            continue; /* in case another ea has a response */
+        }
+
         { /** dummy block to preserve indentation; reformat later */
+            if (ea_list != orig)
+                res_log(NULL,LOG_DEBUG,"libsres: "" found response in ea %p",
+                        ea_list);
             *answer = ea_list->ea_response;
             *answer_length = ea_list->ea_response_length;
             res_log(NULL, LOG_DEBUG,
                     "libsres: ""get_response got %zd bytes on socket %d",
                     *answer_length, ea_list->ea_socket);
 
-            /*
-             * don't clone all when we just need one. temporarily set
-             * number of nameservers to 1 before cloning.
-             */
-            if (ea_list->ea_ns->ns_number_of_addresses > 1) {
-                save_count = ea_list->ea_ns->ns_number_of_addresses;
-                ea_list->ea_ns->ns_number_of_addresses = 1;
-            }
-            if (SR_UNSET !=
-                (retval = clone_ns(respondent, ea_list->ea_ns)))
+            retval = _clone_respondent(ea_list, respondent);
+
+            if (SR_UNSET != retval)
                 return retval;
-            if (save_count > 0) /* restore original count */
-                ea_list->ea_ns->ns_number_of_addresses = save_count;
 
-            /** if response wasn't from first address, fixup respondent */
-            if (ea_list->ea_which_address != 0) {
-                memcpy(((*respondent)->ns_address[0]),
-                       ea_list->ea_ns->ns_address[ea_list->
-                                                  ea_which_address],
-                       sizeof(struct sockaddr_storage));
-            }
             ea_list->ea_response = NULL;
             ea_list->ea_response_length = 0;
             return SR_IO_GOT_ANSWER;
         }
     }
-    return SR_IO_UNSET;
+
+    if (0 == retries) {
+        res_log(NULL, LOG_DEBUG, "libsres: ""*** no answer and no retries!");
+        retval = SR_IO_NO_ANSWER;
+    }
+
+    return retval;
 }
 
 size_t
@@ -1147,7 +1362,7 @@ res_io_read_udp(struct expected_arrival *arrival)
      * Cancel this source 
      */
     res_io_cancel_source(arrival);
-    res_log(NULL, LOG_ERR, "libsres: ""Closing socket %d, %s",
+    res_log(NULL, LOG_INFO, "libsres: ""Closing socket %d, %s",
             arrival->ea_socket, (ret_val == 0) ?
             "socket shutdown" : "read_udp failed");
 
@@ -1180,15 +1395,13 @@ res_switch_to_tcp(struct expected_arrival *ea)
         ea->ea_socket = INVALID_SOCKET;
     }
     ea->ea_remaining_attempts = ea->ea_ns->ns_retry+1;
-    set_alarm(&ea->ea_next_try, 0);
-
-    set_alarm(&ea->ea_cancel_time, res_get_timeout(ea->ea_ns));
+    set_alarms(ea, 0, res_get_timeout(ea->ea_ns));
 }
 
 /*
  * switch all ea entries in the chain to tcp.
  *
- * unline res_switch_to_tcp, which is used during processing, this
+ * unlike res_switch_to_tcp, which is used during processing, this
  * function is intended to be called BEFORE processing starts. Thus
  * the retry count and next/cancel timers are not touched.
  */
@@ -1397,6 +1610,8 @@ res_io_accept(int transaction_id, fd_set *pending_desc,
 
     if (ret_val == SR_IO_UNSET)
         return SR_IO_NO_ANSWER_YET;
+    else if (ret_val == SR_IO_NO_ANSWER)
+        return SR_IO_NO_ANSWER;
     else
         return SR_IO_GOT_ANSWER;
 }
@@ -1431,6 +1646,21 @@ res_io_cancel_all(void)
     }
 }
 
+void
+res_print_server(struct name_server *ea_ns, int i)
+{
+    struct sockaddr_in *s =
+        (struct sockaddr_in *) ((ea_ns->ns_address[i]));
+    char            buf[INET6_ADDRSTRLEN + 1];
+    const char     *addr = NULL;
+    size_t	    buflen = sizeof(buf);
+    if (AF_INET == ea_ns->ns_address[i]->ss_family)
+        INET_NTOP(AF_INET, (struct sockaddr *)s, sizeof(s), buf, buflen,
+                  addr);
+    res_log(NULL, LOG_DEBUG, "libsres: ""   Nameserver: %s", 
+            addr ? addr : "");
+}
+
 void
 res_print_ea(struct expected_arrival *ea)
 {
@@ -1438,6 +1668,7 @@ res_print_ea(struct expected_arrival *ea)
     char            buf[INET6_ADDRSTRLEN + 1];
     const char     *addr = NULL;
     size_t	    buflen = sizeof(buf);
+    struct timeval  now,when_next, when_cancel;
 
         struct sockaddr_in *s =
             (struct sockaddr_in *) ((ea->ea_ns->ns_address[i]));
@@ -1456,15 +1687,41 @@ res_print_ea(struct expected_arrival *ea)
             port = s->sin_port;
         }
 
-        res_log(NULL, LOG_DEBUG, "libsres: "
-                "  ea %p Socket: %d, Stream: %d, Nameserver: %s/(%d)",
-                ea, ea->ea_socket, ea->ea_using_stream, addr ? addr : "",
-                ntohs(port));
-        res_log(NULL, LOG_DEBUG, "libsres: "
-                "  Remaining retries: %d, Next try %ld.%ld, Cancel at %ld.%ld",
-                ea->ea_remaining_attempts, ea->ea_next_try.tv_sec,
-                ea->ea_next_try.tv_usec, ea->ea_cancel_time.tv_sec,
-                ea->ea_cancel_time.tv_usec);
+        gettimeofday(&now, NULL);
+        timersub(&ea->ea_next_try, &now, &when_next);
+        timersub(&ea->ea_cancel_time, &now, &when_cancel);
+
+        if (ea->ea_remaining_attempts < 0) { 
+            res_log(NULL, LOG_DEBUG, "libsres: ""  ea %p "
+#ifdef EA_EXTRA_DEBUG
+                    "%s "
+#endif
+                    "Socket: %d, Nameserver: %s:%d, no more retries", ea,
+#ifdef EA_EXTRA_DEBUG
+                    ea->name,
+#endif
+                    ea->ea_socket, addr ? addr : "", ntohs(port));
+        } else {
+            res_log(NULL, LOG_DEBUG, "libsres: " "  ea %p "
+#ifdef EA_EXTRA_DEBUG
+                    "{%s %s(%d) %s(%d)} "
+#endif
+                    "Socket: %d, Stream: %d, Nameserver: %s:%d", ea,
+#ifdef EA_EXTRA_DEBUG
+                    ea->name, p_class(ea->ea_class_h), ea->ea_class_h,
+                    p_type(ea->ea_type_h), ea->ea_type_h,
+#endif
+                    ea->ea_socket, ea->ea_using_stream, addr ? addr : "",
+                    ntohs(port));
+            res_log(NULL, LOG_DEBUG, "libsres: "
+                    "  Remaining retries: %d, "
+                    "Next try %ld.%ld (%ld.%ld), Cancel at %ld.%ld (%ld.%ld)",
+                    ea->ea_remaining_attempts, ea->ea_next_try.tv_sec,
+                    ea->ea_next_try.tv_usec, when_next.tv_sec,
+                    when_next.tv_usec, ea->ea_cancel_time.tv_sec,
+                    ea->ea_cancel_time.tv_usec, when_cancel.tv_sec,
+                    when_cancel.tv_usec);
+        } 
 }
 
 void
@@ -1592,6 +1849,11 @@ res_async_query_create(const char *name, const u_int16_t type_h,
             ret_val = SR_IO_MEMORY_ERROR;
             break; /* fatal, bail */
         }
+#ifdef EA_EXTRA_DEBUG
+        new_ea->name = strdup(name);
+        new_ea->ea_type_h = type_h;
+        new_ea->ea_class_h = class_h;
+#endif
 
         /** add to list */
         if (NULL != head) {
