@@ -1,19 +1,10 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef RES_MKQUERY_H
 #define RES_MKQUERY_H
 
-
-/*
- * should be in resolv.h; define it if not. this value is from
- * the OS X header (linux uses a different value)
- */
-#ifndef RES_USE_EDNS0
-#define RES_USE_EDNS0   0x40000000
-#endif
-
 #ifndef T_OPT
 #define T_OPT   41
 #endif
