@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -24,7 +24,7 @@
  * Contains implementation for storage and retrieval functions for the validator
  * rrset cache.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_support.h"
@@ -220,19 +220,25 @@ get_cached_rrset(struct val_query_chain *matched_q,
     while (!done) {
 
         if (look_for_negative) {
+#ifndef VAL_NO_THREADS
             lk = &proof_rwlock;
             VAL_CACHE_LOCK_INIT(lk, proof_rwlock_init);
             VAL_CACHE_LOCK_SH(lk);
+#endif /* VAL_NO_THREADS */
             answer_head = &unchecked_proofs;
         } else if (type_h == ns_t_ns) {
+#ifndef VAL_NO_THREADS
             lk = &ns_rwlock;
             VAL_CACHE_LOCK_INIT(lk, ns_rwlock_init);
             VAL_CACHE_LOCK_SH(lk);
+#endif /* VAL_NO_THREADS */
             answer_head = &unchecked_ns_info;
         } else {
+#ifndef VAL_NO_THREADS
             lk = &ans_rwlock;
             VAL_CACHE_LOCK_INIT(lk, ans_rwlock_init);
             VAL_CACHE_LOCK_SH(lk);
+#endif /* VAL_NO_THREADS */
             answer_head = &unchecked_answers;
         }
 
