@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -14,7 +14,7 @@
  *
  * See RFC 2537, RFC 3110, RFC 4034 Appendix B.1, RFC 2536
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include <openssl/bn.h>
