@@ -1,13 +1,13 @@
 
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
  * DESCRIPTION
  * Contains the implementation for the logging functionality in libval
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_cache.h"
@@ -155,16 +155,26 @@ val_log_rrsig_rdata(const val_context_t * ctx, int level,
 {
     char            ctime_buf1[1028], ctime_buf2[1028];
     char            buf[1028];
+    struct timeval  tv_sig1, tv_sig2;
+
     if (rdata) {
         if (!prefix)
             prefix = "";
+
+        memset(&tv_sig1, 0, sizeof(tv_sig1));
+        memset(&tv_sig2, 0, sizeof(tv_sig2));
+        tv_sig1.tv_sec = rdata->sig_expr; 
+        tv_sig2.tv_sec = rdata->sig_incp; 
+
+        GET_TIME_BUF((const time_t *)(&tv_sig1.tv_sec), ctime_buf1);
+        GET_TIME_BUF((const time_t *)(&tv_sig2.tv_sec), ctime_buf2);
+
         val_log(ctx, level, "%s Type=%d Algo=%d[%s] Labels=%d OrgTTL=%d "
                 "SigExp=%s SigIncp=%s KeyTag=%d[0x %04x] Signer=%s Sig=%s",
                 prefix, rdata->algorithm,
                 get_algorithm_string(rdata->algorithm), rdata->labels,
                 rdata->orig_ttl,
-                GET_TIME_BUF((const time_t *) (&(rdata->sig_expr)), ctime_buf1),
-                GET_TIME_BUF((const time_t *) (&(rdata->sig_incp)), ctime_buf2),
+                ctime_buf1, ctime_buf2,
                 rdata->key_tag, rdata->key_tag, rdata->signer_name,
                 get_base64_string(rdata->signature, rdata->signature_len,
                                   buf, 1024));
@@ -402,8 +412,14 @@ p_query_status(int err)
             return "Q_INIT";
         case Q_SENT:
             return "Q_SENT";
-        case Q_WAIT_FOR_GLUE:
+        case Q_ANSWERED:
+            return "Q_ANSWERED";
+        case (Q_WAIT_FOR_A_GLUE|Q_WAIT_FOR_AAAA_GLUE):
+        case Q_WAIT_FOR_A_GLUE:
+        case Q_WAIT_FOR_AAAA_GLUE:
             return "Q_WAIT_FOR_GLUE";
+        case Q_QUERY_ERROR:
+            return "Q_QUERY_ERROR";
         case Q_RESPONSE_ERROR:
             return "Q_RESPONSE_ERROR";
         case Q_WRONG_ANSWER:
@@ -750,8 +766,7 @@ val_log_filep(val_log_t * logp, const val_context_t * ctx, int level,
             tm->tm_hour, tm->tm_min, tm->tm_sec);
     vsnprintf(&buf[19], sizeof(buf) - 21, template, ap);
 
-    fprintf(logp->opt.file.fp, buf);
-    fprintf(logp->opt.file.fp, "\n");
+    fprintf(logp->opt.file.fp, "%s\n", buf);
     fflush(logp->opt.file.fp);
 }
 
