@@ -1,12 +1,12 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
  * DESCRIPTION:
  * Contains functions that interface to the signature verification routines.
  */
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_support.h"
@@ -96,6 +96,7 @@ val_sigverify(val_context_t * ctx,
               int clock_skew)
 {
     struct timeval  tv;
+    struct timeval  tv_sig;
 
     /** Inputs to this function have already been NULL-checked **/
 
@@ -141,15 +142,16 @@ val_sigverify(val_context_t * ctx,
             if (tv.tv_sec < rrsig->sig_incp - clock_skew) {
                 char            currTime[1028];
                 char            incpTime[1028];
-                char            *timeptr1 = NULL;
-                char            *timeptr2 = NULL;
-                memset(currTime, 0, 1028);
-                memset(incpTime, 0, 1028);
-                timeptr1 = GET_TIME_BUF((const time_t *) (&(tv.tv_sec)), currTime);
-                timeptr2 = GET_TIME_BUF((const time_t *) (&(rrsig->sig_incp)), incpTime);
+
+                memset(&tv_sig, 0, sizeof(tv_sig));
+                tv_sig.tv_sec = rrsig->sig_incp;
+
+                GET_TIME_BUF((const time_t *)(&tv.tv_sec), currTime);
+                GET_TIME_BUF((const time_t *)(&tv_sig.tv_sec), incpTime);
+
                 val_log(ctx, LOG_INFO,
                         "val_sigverify(): Signature not yet valid. Current time (%s) is less than signature inception time (%s).",
-                        timeptr1, timeptr2);
+                        currTime, incpTime);
                 *sig_status = VAL_AC_RRSIG_NOTYETACTIVE;
                 return 0;
             } else {
@@ -163,15 +165,18 @@ val_sigverify(val_context_t * ctx,
             if (tv.tv_sec > rrsig->sig_expr + clock_skew) {
                 char            currTime[1028];
                 char            exprTime[1028];
-                char            *timeptr1 = NULL;
-                char            *timeptr2 = NULL;
-                memset(currTime, 0, 1028);
-                memset(exprTime, 0, 1028);
-                timeptr1 = GET_TIME_BUF((const time_t *) (&(tv.tv_sec)), currTime);
-                timeptr2 = GET_TIME_BUF((const time_t *) (&(rrsig->sig_expr)), exprTime);
+
+                memset(&tv_sig, 0, sizeof(tv_sig));
+                tv_sig.tv_sec = rrsig->sig_expr;
+
+                memset(currTime, 0, sizeof(currTime));
+                memset(exprTime, 0, sizeof(exprTime));
+                GET_TIME_BUF((const time_t *)(&tv.tv_sec), currTime);
+                GET_TIME_BUF((const time_t *)(&tv_sig.tv_sec), exprTime);
+
                 val_log(ctx, LOG_INFO,
                         "val_sigverify(): Signature expired. Current time (%s) is greater than signature expiration time (%s).",
-                        timeptr1, timeptr2);
+                        currTime, exprTime);
                 *sig_status = VAL_AC_RRSIG_EXPIRED;
                 return 0;
             } else {
@@ -450,7 +455,8 @@ do_verify(val_context_t * ctx,
           val_astatus_t * sig_status,
           struct rrset_rec *the_set,
           struct val_rr_rec *the_sig,
-          val_dnskey_rdata_t * the_key, int is_a_wildcard)
+          val_dnskey_rdata_t * the_key, int is_a_wildcard,
+          u_int32_t flags)
 {
     /*
      * Use the crypto routines to verify the signature
@@ -504,9 +510,14 @@ do_verify(val_context_t * ctx,
 
     rrsig_rdata.next = NULL;
 
-    get_clock_skew(ctx, zone_n, &clock_skew, &ttl_x);
-    /* the state is valid for only as long as the policy validity period */
-    SET_MIN_TTL(the_set->rrs_ttl_x, ttl_x);
+    if (flags & VAL_QUERY_IGNORE_SKEW) {
+        clock_skew = -1;
+        val_log(ctx, LOG_DEBUG, "do_verify(): Ignoring clock skew"); 
+    } else {
+        get_clock_skew(ctx, zone_n, &clock_skew, &ttl_x);
+        /* the state is valid for only as long as the policy validity period */
+        SET_MIN_TTL(the_set->rrs_ttl_x, ttl_x);
+    }
 
     /*
      * Perform the verification 
@@ -624,7 +635,8 @@ check_label_count(struct rrset_rec *the_set,
 void
 verify_next_assertion(val_context_t * ctx,
                       struct val_digested_auth_chain *as,
-                      struct val_digested_auth_chain *the_trust)
+                      struct val_digested_auth_chain *the_trust,
+                      u_int32_t flags)
 {
     struct rrset_rec *the_set;
     struct val_rr_rec  *the_sig;
@@ -729,7 +741,7 @@ verify_next_assertion(val_context_t * ctx,
             is_verified = do_verify(ctx, signby_name_n,
                       &nextrr->rr_status,
                       &the_sig->rr_status,
-                      the_set, the_sig, &dnskey, is_a_wildcard);
+                      the_set, the_sig, &dnskey, is_a_wildcard, flags);
 
             /*
              * There might be multiple keys with the same key tag; set this as
@@ -757,9 +769,8 @@ verify_next_assertion(val_context_t * ctx,
         if (nextrr == NULL) {
             val_log(ctx, LOG_INFO, "verify_next_assertion(): Could not link this RRSIG to a DNSKEY");
             SET_STATUS(as->val_ac_status, the_sig, VAL_AC_DNSKEY_NOMATCH);
-        } 
 
-        if (as->val_ac_status == VAL_AC_TRUST ||
+        } else if (as->val_ac_status == VAL_AC_TRUST ||
             (is_verified && 
                      nextrr->rr_status == VAL_AC_TRUST_POINT)) {
             /* we've verified a trust anchor */
